#!/bin/bash
#
# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
  cat <<EOF
USAGE: `basename ${0}` <product_dir> <e10|e13|e14|e15|e17|e19|c2|c5|c7> <debug|opt|prof> [tar]
EOF
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
  qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
  use_apple_clang=false
  for qual in "${qlist[@]}"; do
    case ${qual} in
      e1[01245])
        cxxflg="-std=c++14 -Wno-deprecated-declarations"
        cqual=$qual
        [[ "${OS1}" = Darwin ]] && toolset=darwin
        ;;
      e13)   # apple clang
        cxxflg="-std=c++14 -Wno-deprecated-declarations"
	      use_apple_clang=true
        toolset=clang
        cqual=$qual
        ;;
      e1[79])
        cxxflg="-std=c++17 -Wno-deprecated-declarations"
        cqual=$qual
        [[ "${OS1}" = Darwin ]] && toolset=darwin
        ;;
      c[257])
        cxxflg="-std=c++17 -Wno-deprecated-declarations -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
        toolset=clang
        cqual=$qual
        ;;
      *)
        ssi_die "Unrecognized qualifier ${qual}"
    esac
  done
}

unpack_and_apply_patches() {
  cd "${sourcedir}"
  tar xf "${tardir}/${pkgtarname}.tar.bz2" || \
    ssi_die "Unable to untar source tarball ${tardir}/${pkgtarname}.tar.bz2"
  patches=(${patchdir}/boost-${origpkgdotver//./_}.patch)
  for patch in "${patches[@]}"; do
    [[ -r "${patch}" ]] || continue # Don't care if there isn't one this time.
    if grep -e '^\(---\|+++\) [ab]/include/' ${patch} >/dev/null 2>&1; then
      pval=2
    elif grep -e '^\(---\|+++\) [ab]/' ${patch} >/dev/null 2>&1; then
      pval=1
    else
      pval=0
    fi
    patch ${DEBUG_PATCHES:+--verbose} -p${pval} < "${patch}" || ssi_die "application of patch ${patch} failed from `pwd`."
  done
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=boost
origpkgver=v1_68_0
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_10
origpkgdotver=`echo ${origpkgver#v} | sed -e 's/_/./g'`
pkgdotver=`echo ${pkgver#v} | sed -e 's/_/./g'`
pkgtarver=`echo ${origpkgver#v}`
pkgtarname=boost_${pkgtarver}
toolset=gcc

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case $extraqual in
  debug)
    variant=debug
    ;;
  prof)
    variant=profile
    ;;
  opt|*)
    variant=release
esac

set_qualifiers

if [ ${OS1} = "Darwin" ]
then
  cxxflg="${cxxflg} -fvisibility-inlines-hidden"
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

set -x

# This will be distributed for debugging
# Apply any patches.
unpack_and_apply_patches

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd "${sourcedir}/${pkgtarname}"

./bootstrap.sh --prefix="${pkgdir}" --with-toolset=${toolset} --without-icu --without-libraries=python,mpi

if [[ ${toolset} = darwin ]]; then
  # Need to tweak jam file.
  perl -wni\~ -e 'print; m&^project.*<toolset>darwin& and print <<EOF;
# Override the compiler used by the darwin toolset.
using gcc : : $ENV{GCC_FQ_DIR}/bin/g++ : root=$ENV{GCC_FQ_DIR} ;
EOF
' project-config.jam
  [[ -s project-config.jam ]] && rm -f project-config.jam~
fi

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

# --debug-building  provides verbose output
# -d+2 echoes build commands
./b2 ${DEBUG_BUILD:+--debug-building -d+2} \
  -j ${num_cores} \
  --build-dir=${blddir}/${pkgtarname} \
  --stagedir=${blddir}/${pkgtarname}/stage \
  --ignore-site-config \
  --disable-icu \
  toolset=${toolset:-gcc} \
  threading=multi \
  link=shared \
  "${cxxflg:+cxxflags=${cxxflg}}" \
  ${variant:+variant=${variant}} install || \
  ssi_die "ERROR: Build failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# Clean up debris from source area.
rm -rf "${sourcedir}/${pkgtarname}/"{b2,bjam,bootstrap.log,project-config.jam,tools/build/src/engine/{bin.*,bootstrap}}

# real ups declare
"${SSIBUILDSHIMS_DIR}/bin/declare_product" "${product_dir}" ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "${package} is installed at ${BOOST_FQ_DIR}"
echo "BOOST_LIB is ${BOOST_LIB}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0

